/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.1.1)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2006 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: real.cpp,v 1.9 2006/03/08 14:16:31 cxsc Exp $ */

#include "real.hpp"
#include "ioflags.hpp"

namespace cxsc {

//----------------------------------------------------------------------------
// MakeHexReal - erstellt aus den binaer angegebenen Einzelteilen einer
//               IEEE 64-bit Gleitkommazahl eine solche, hierbei ist
//                 sign         Flag 'zu erzeugener real ist negativ'
//                 expo         Exponent (11 Bit 0x000 - 0x7FF)
//                 manthigh     obere  20 Bit der Mantisse (Bit 32 - 53)
//                 mantlow      untere 32 Bit der Mantisse (Bit  0 - 31)
//
//             - die Mantisse ist normalisiert dargestellt, d.h.
//               implizit wird noch ein Bit 1 als Vorkommastelle
//               vorangestellt.
//               Eine Ausnahme bilden die Zahlen mit Exponent = 0,
//               oder Exponent = 0x7FF (alle Exponentenbits = 1)
//
const real& MakeHexReal(int sign, unsigned int expo, a_btyp manthigh, a_btyp mantlow)
{
  static real a;
  ((a_btyp*)&a)[LOWREAL]   = mantlow,
  ((a_btyp*)&a)[HIGHREAL]  = manthigh & 0xFFFFFL,
  ((a_btyp*)&a)[HIGHREAL] |= ((a_btyp) (expo & 0x7FF)) << 20,
  ((a_btyp*)&a)[HIGHREAL] |= (sign ? 0x80000000L : 0x00000000);
  return a;
}

const real MinReal      = MakeHexReal(0, 0x001, 0x00000L, 0x00000000L);
const real minreal      = MakeHexReal(0, 0x000, 0x00000L, 0x00000001L);
// Blomquist, 26.09.02; minreal = smallest positive real number.
const real MaxReal      = MakeHexReal(0, 0x7FE, 0xFFFFFL, 0xFFFFFFFFL);
const real Infinity     = MakeHexReal(0, 0x7FF, 0x00000L, 0x00000000L);
const real SignalingNaN = MakeHexReal(1, 0x7FF, 0x80000L, 0x00000000L);
const real QuietNaN     = MakeHexReal(0, 0x7FF, 0x00000L, 0x00000001L);

} // namespace cxsc
